# frozen_string_literal: true

require "gitlab_release_notes/version"
require 'tzinfo'
require 'git'
require 'gitlab'

module GitlabReleaseNotes
  class Error < StandardError; end
  class Client
    JIRA_URL = ENV['JIRA_URL']
    JIRA_IDS_REGEX = ENV['JIRA_IDS_REGEX'] || '(\[([A-Z]{1,}-[0-9]{1,})\])(.*)'
    TIME_ZONE = ENV['TZ'] || 'Africa/Cairo'
    ELIGABLE_COMMITS_REGEX = ENV['ELIGABLE_COMMITS_REGEX'] || '(\[[A-Z]{1,}-[0-9]{1,}\]((?!Merge.*branch).)*$)'
    APP_VERSIONS_REGEX = ENV['APP_VERSIONS_REGEX'] || 'v[0-9]{1,}\.[0-9]{1,}\.[0-9]{1,}'
    RELEASE_NOTES_HEADER = ENV['RELEASE_NOTES_HEADER'] || '##### Release Notes'
  
    def log_filter(logs)
      app_changelog = []
      logs.each do |l|
        if l.message.match(/#{ELIGABLE_COMMITS_REGEX}/)
          app_changelog.append(l.message.gsub(/#{JIRA_IDS_REGEX}/,
                                              "\\1(#{JIRA_URL}/browse/\\2)\\3<br>"))
        end
      end
      app_changelog
    end
  
    def tags_lister(repository)
      app_versions = []
      repository.tags.each do |t|
        app_versions.append(t.name) \
                       if t.name.match(/#{APP_VERSIONS_REGEX}/)
      end
      app_versions
    end

    def current_time
      tz = TZInfo::Timezone.get(TIME_ZONE.to_s)
      tz.now
    end

    def prepare_release_notes(repository_path)
      repo = Git.open(repository_path)
      release_notes = []
      app_versions = tags_lister(repo)
      if app_versions.length == 1
        release_notes = log_filter(repo.log)
      else
        eligable_versions = app_versions.last(2)
        release_notes = log_filter(repo.log.between(eligable_versions[0], eligable_versions[1]))
      end
      "#{RELEASE_NOTES_HEADER} (#{current_time})<br><hr> " + release_notes.uniq.join('')
    end

    def publish_release_notes(repository_id, release_tag, repository_path)
      release_notes = prepare_release_notes(repository_path)
      begin
        Gitlab.create_release(repository_id, release_tag, release_notes)
        "[#{release_tag}] Release notes have been created"
      rescue Gitlab::Error::Conflict
        Gitlab.update_release(repository_id, release_tag, release_notes)
        "[#{release_tag}] Release notes have been updated"
      end
    end
  end
end
